/* eslint-disable */
//notation: js file can only use this kind of comments
//since comments will cause error when use in webview.loadurl,
//comments will be remove by java use regexp
(function () {
  if (window.WebViewJavascriptBridge || window.NativeJsbridgeInjectStart) {
    console.log('==========WebViewJavascriptBridge 已存在==========');
    return;
  }
  const list = [];
  window.bridgeLog = function (name, time) {
    const last = list.length > 0 ? list[list.length - 1] : null;
    const duration = last ? time - last.time : 0;
    const first = list.length > 0 ? list[0] : null;
    const total = first ? time - first.time : 0;
    list.push({ name, time, duration, total });
    console.log(name, time, duration, total);
  };
  window.printBridgeLog = function () {
    list.forEach(e => {
      console.log(e.name, e.time, e.duration, e.total);
    });
  };
  let messagingIframe;
  let sendMessageQueue = [];
  let receiveMessageQueue = [];
  const messageHandlers = {};

  const CUSTOM_PROTOCOL_SCHEME = 'yy';
  const QUEUE_HAS_MESSAGE = '__QUEUE_MESSAGE__/';

  const responseCallbacks = {};
  let uniqueId = 1;

  const base64encodechars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  function base64encode(str) {
    if (str === undefined) {
      return str;
    }

    let out, i;
    let c1, c2, c3;
    const len = str.length;
    i = 0;
    out = '';
    while (i < len) {
      c1 = str.charCodeAt(i++) & 0xff;
      if (i == len) {
        out += base64encodechars.charAt(c1 >> 2);
        out += base64encodechars.charAt((c1 & 0x3) << 4);
        out += '==';
        break;
      }
      c2 = str.charCodeAt(i++);
      if (i == len) {
        out += base64encodechars.charAt(c1 >> 2);
        out += base64encodechars.charAt(((c1 & 0x3) << 4) | ((c2 & 0xf0) >> 4));
        out += base64encodechars.charAt((c2 & 0xf) << 2);
        out += '=';
        break;
      }
      c3 = str.charCodeAt(i++);
      out += base64encodechars.charAt(c1 >> 2);
      out += base64encodechars.charAt(((c1 & 0x3) << 4) | ((c2 & 0xf0) >> 4));
      out += base64encodechars.charAt(((c2 & 0xf) << 2) | ((c3 & 0xc0) >> 6));
      out += base64encodechars.charAt(c3 & 0x3f);
    }
    return out;
  }

  function _createQueueReadyIframe(doc) {
    messagingIframe = doc.createElement('iframe');
    messagingIframe.style.display = 'none';
    doc.documentElement.appendChild(messagingIframe);
  }

  function isAndroid() {
    const ua = navigator.userAgent.toLowerCase();
    const isA = ua.indexOf('android') > -1;
    if (isA) {
      return true;
    }
    return false;
  }

  function isIphone() {
    const ua = navigator.userAgent.toLowerCase();
    const isIph = ua.indexOf('iphone') > -1;
    if (isIph) {
      return true;
    }
    return false;
  }

  //set default messageHandler
  function init(messageHandler) {
    // eslint-disable-next-line no-undef
    if (WebViewJavascriptBridge._messageHandler) {
      throw new Error('WebViewJavascriptBridge.init called twice');
    }
    // eslint-disable-next-line no-undef
    WebViewJavascriptBridge._messageHandler = messageHandler;
    const receivedMessages = receiveMessageQueue;
    receiveMessageQueue = null;
    for (let i = 0; i < receivedMessages.length; i++) {
      _dispatchMessageFromNative(receivedMessages[i]);
    }
  }

  function send(data, responseCallback) {
    _doSend(
      {
        data: data,
      },
      responseCallback
    );
  }

  function registerHandler(handlerName, handler) {
    messageHandlers[handlerName] = handler;
  }

  function callHandler(handlerName, data, responseCallback) {
    _doSend(
      {
        handlerName: handlerName,
        data: data,
      },
      responseCallback
    );
  }

  //sendMessage add message, 触发native处理 sendMessage
  function _doSend(message, responseCallback) {
    if (responseCallback) {
      const callbackId = 'cb_' + uniqueId++ + '_' + new Date().getTime();
      responseCallbacks[callbackId] = responseCallback;
      message.callbackId = callbackId;
    }

    sendMessageQueue.push(message);
    messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;
  }

  // 提供给native调用,该函数作用:获取sendMessageQueue返回给native,由于android不能直接获取返回的内容,所以使用url shouldOverrideUrlLoading 的方式返回内容
  function _fetchQueue() {
    const messageQueueString = JSON.stringify(sendMessageQueue);
    sendMessageQueue = [];
    //add by hq
    if (isIphone()) {
      return messageQueueString;
      //android can't read directly the return data, so we can reload iframe src to communicate with java
    } else if (isAndroid()) {
      messagingIframe.src =
        CUSTOM_PROTOCOL_SCHEME + '://return/_fetchQueue/' + encodeURIComponent(messageQueueString);
    }
  }

  //提供给native使用,
  function _dispatchMessageFromNative(messageJSON) {
    setTimeout(function () {
      const message = JSON.parse(messageJSON);
      let responseCallback;
      //java call finished, now need to call js callback function
      if (message.responseId) {
        responseCallback = responseCallbacks[message.responseId];
        if (!responseCallback) {
          return;
        }
        responseCallback(message.responseData);
        delete responseCallbacks[message.responseId];
      } else {
        //直接发送
        if (message.callbackId) {
          const callbackResponseId = message.callbackId;
          responseCallback = function (responseData) {
            _doSend({
              responseId: callbackResponseId,
              responseData: responseData,
            });
          };
        }

        // eslint-disable-next-line no-undef
        let handler = WebViewJavascriptBridge._messageHandler;
        if (message.handlerName) {
          handler = messageHandlers[message.handlerName];
        }
        //查找指定handler
        try {
          handler(message.data, responseCallback);
        } catch (exception) {
          if (typeof console != 'undefined') {
            console.log(
              'WebViewJavascriptBridge: WARNING: javascript handler threw.',
              message,
              exception
            );
          }
        }
      }
    });
  }

  //提供给native调用,receiveMessageQueue 在会在页面加载完后赋值为null,所以
  function _handleMessageFromNative(messageJSON) {
    console.log(messageJSON);
    if (receiveMessageQueue) {
      receiveMessageQueue.push(messageJSON);
    } else {
      _dispatchMessageFromNative(messageJSON);
    }
  }

  function initBridge() {
    window.bridgeLog('========== 进入 initBridge ==========', Date.now());
    const doc = document;
    _createQueueReadyIframe(doc);
    window.bridgeLog('========== 创建 Iframe完成 ==========', Date.now());
    const WebViewJavascriptBridge = (window.WebViewJavascriptBridge = {
      init: init,
      send: send,
      registerHandler: registerHandler,
      callHandler: callHandler,
      _fetchQueue: _fetchQueue,
      _handleMessageFromNative: _handleMessageFromNative,
    });
    window.bridgeLog('========== 创建 WebViewJavascriptBridge完成 ==========', Date.now());
    const readyEvent = doc.createEvent('Events');
    readyEvent.initEvent('WebViewJavascriptBridgeReady');
    readyEvent.bridge = WebViewJavascriptBridge;
    window.bridgeLog('========== 开始下发WebViewJavascriptBridge完成事件  ==========', Date.now());
    doc.dispatchEvent(readyEvent);
    window.bridgeLog('========== 下发完成WebViewJavascriptBridge完成事件  ==========', Date.now());
    console.log('jsbridge inject success');
  }

  function doDetectDoc(callback) {
    if (
      document &&
      document.createElement &&
      document.documentElement &&
      document.documentElement.appendChild
    ) {
      window.bridgeLog('==========doDetectDoc 成功==========', Date.now());
      callback();
      return;
    }
    setTimeout(function () {
      doDetectDoc(callback);
    }, 4);
  }

  let isDoingDetect = false;

  function detectDoc(callback) {
    if (isDoingDetect) {
      return;
    }
    window.bridgeLog('==========进入 detectDoc==========', Date.now());
    isDoingDetect = true;
    doDetectDoc(callback);
  }

  function domReady(callback) {
    window.bridgeLog('==========domReady 已存在==========', Date.now());
    if (!document) {
      setTimeout(function () {
        domReady(callback);
      }, 0);
      return false;
    }
    window.bridgeLog('========== readyState complete==========' + document.readyState, Date.now());
    if (document.readyState === 'complete') {
      window.bridgeLog('========== readyState complete========== complete', Date.now());
      callback();
      return;
    }
    window.bridgeLog('========== 开始detectDoc==========' + document.readyState, Date.now());
    detectDoc(callback);
    /*window.addEventListener('DOMContentLoaded', function () {
      window.bridgeLog('========== DOMContentLoaded ==========', Date.now());
      callback();
    });*/
  }
  domReady(initBridge);
})();
